@@ -4,7 +4,8 @@ from .wrapper import TabularData
 
 
 __all__ = [
-    "TabularData", "TabularDataset", "TaskTypes", "ColumnTypes",
+    "TabularData", "TabularDataset",
+    "TaskTypes", "ColumnTypes", "DataTuple",
     "SplitResult", "DataSplitter", "KFold", "KRandom",
     "ImbalancedSampler", "LabelCollators", "DataLoader"
 ]
