@@ -21,7 +21,7 @@ class TabularData(DataBase):
     def __init__(self,
                  *,
                  task_type: TaskTypes = None,
-                 label_name: str = "label",
+                 label_name: str = None,
                  string_label: Union[bool, None] = None,
                  numerical_label: Union[bool, None] = None,
                  categorical_label: Union[bool, None] = None,
@@ -63,7 +63,8 @@ class TabularData(DataBase):
         self._numerical_threshold = numerical_threshold
         self._is_file = self._is_arr = False
         self._num_classes = None
-        self._label_idx = self._skip_first = self._delim = None
+        self._label_idx = self._has_column_names = None
+        self._delim = self._quote_char = None
         self._raw = self._converted = self._processed = None
         self._recognizers = self._converters = self._processors = None
         self._verbose_level = verbose_level
@@ -304,7 +305,7 @@ class TabularData(DataBase):
                     processor = self._processors[-1] = processor_dict[method]([]).fit(converted_labels)
                 with timing_context(self, "process with processor"):
                     processed_labels = processor.process(converted_labels)
-        if self.task_type is TaskTypes.CLASSIFICATION:
+        if self.task_type is TaskTypes.CLASSIFICATION and converted_labels is not None:
             converted_labels = converted_labels.astype(np_int_type)
             processed_labels = processed_labels.astype(np_int_type)
         self._converted = DataTuple(converted_x, converted_labels)
@@ -316,11 +317,13 @@ class TabularData(DataBase):
     def _read_from_file(self,
                         file_path: str,
                         *,
-                        label_idx: int = -1,
-                        skip_first: bool = None,
+                        label_idx: int = None,
+                        has_column_names: bool = None,
+                        quote_char: str = None,
                         delim: str = None) -> "TabularData":
         self._is_file = True
-        self._label_idx, self._skip_first, self._delim = label_idx, skip_first, delim
+        self._label_idx, self._has_column_names = label_idx, has_column_names
+        self._delim, self._quote_char = delim, quote_char
         with timing_context(self, "_read_file"):
             x, y = self.read_file(file_path)
         self._raw = DataTuple.with_transpose(x, y)
@@ -378,34 +381,88 @@ class TabularData(DataBase):
 
     def _get_raw(self,
                  x: Union[str, data_type],
-                 y: data_type = None) -> DataTuple:
+                 y: data_type = None,
+                 *,
+                 contains_labels: bool = True) -> DataTuple:
         if self._is_file:
             if isinstance(x, str):
                 if y is not None:
                     raise ValueError(f"x refers to file_path but y is still provided ({y}), which is illegal")
-                x, y = self.read_file(x)
+                x, y = self.read_file(x, contains_labels=contains_labels)
         return DataTuple.with_transpose(x, y)
 
     # API
 
     def read_file(self,
-                  file_path: str) -> Tuple[raw_data_type, raw_data_type]:
+                  file_path: str,
+                  *,
+                  contains_labels: bool = True) -> Tuple[raw_data_type, raw_data_type]:
         ext = os.path.splitext(file_path)[1][1:]
         set_default = lambda n, default: n if n is not None else default
         if ext == "txt":
-            skip_first, delim = map(set_default, [self._skip_first, self._delim], [False, " "])
+            has_column_names, delim, quote_char = map(
+                set_default,
+                [self._has_column_names, self._delim, self._quote_char],
+                [False, " ", None]
+            )
         elif ext == "csv":
-            skip_first, delim = map(set_default, [self._skip_first, self._delim], [True, ","])
+            has_column_names, delim, quote_char = map(
+                set_default,
+                [self._has_column_names, self._delim, self._quote_char],
+                [True, ",", '"']
+            )
         else:
             raise NotImplementedError(f"file type '{ext}' not recognized")
         with open(file_path, "r") as f:
-            if skip_first:
-                f.readline()
-            data = [["nan" if not elem else elem for elem in line.strip().split(delim)] for line in f]
-        if self._label_idx is None:
-            if len(data[0]) != len(self._raw.x[0]):
+            first_row = None
+            if has_column_names:
+                first_row = column_names = f.readline().strip().split(delim)
+                self._column_names = {i: name for i, name in enumerate(column_names)}
+            data = []
+            for line in f:
+                line = ["nan" if not elem else elem for elem in line.strip().split(delim)]
+                if quote_char is not None:
+                    startswith_quote = [elem.startswith(quote_char) for elem in line]
+                    endswith_quote = [elem.endswith(quote_char) for elem in line]
+                    merge_start, merge_intervals = None, []
+                    for i, (startswith, endswith) in enumerate(zip(startswith_quote, endswith_quote)):
+                        if startswith and not endswith:
+                            merge_start = i
+                            continue
+                        if endswith and not startswith and merge_start is not None:
+                            merge_intervals.append((merge_start, i + 1))
+                            merge_start = None
+                            continue
+                    idx, new_line = 0, []
+                    for start, end in merge_intervals:
+                        if start > idx:
+                            new_line += line[idx:start]
+                        new_line.append(delim.join(line[start:end]))
+                        idx = end
+                    if idx < len(line):
+                        new_line += line[idx:len(line)]
+                    line = new_line
+                if first_row is None:
+                    first_row = line
+                else:
+                    assert len(first_row) == len(line), "num_features are not identical"
+                data.append(line)
+        if not contains_labels:
+            if len(data[0]) == len(self._raw.x[0]) + 1:
                 raise ValueError("file contains labels but 'contains_labels=False' passed in")
             return data, None
+
+        if self._column_names is None or self.label_name is None:
+            if self._label_idx is None:
+                self._label_idx = -1
+        else:
+            reverse_column_names = dict(map(reversed, self._column_names.items()))
+            self._label_idx = reverse_column_names.get(self.label_name)
+            if self._label_idx is None:
+                raise ValueError(
+                    f"'{self.label_name}' is not included in column names "
+                    f"({[self._column_names[i] for i in range(len(self._column_names))]})"
+                )
         if self._label_idx < 0:
             self._label_idx = len(data[0]) + self._label_idx
         x = [line[:self._label_idx] + line[self._label_idx+1:] for line in data]
@@ -414,7 +471,7 @@ class TabularData(DataBase):
 
     def read(self,
              x: Union[str, data_type],
-             y: Union[int, data_type] = -1,
+             y: Union[int, data_type] = None,
              **kwargs) -> "TabularData":
         if isinstance(x, str):
             self._read_from_file(x, label_idx=y, **kwargs)
@@ -445,9 +502,11 @@ class TabularData(DataBase):
 
     def copy_to(self,
                 x: Union[str, data_type],
-                y: data_type = None) -> "TabularData":
+                y: data_type = None,
+                *,
+                contains_labels: bool = True) -> "TabularData":
         copied = copy.copy(self)
-        raw = copied._raw = self._get_raw(x, y)
+        raw = copied._raw = self._get_raw(x, y, contains_labels=contains_labels)
         copied._converted, copied._processed = self._transform(raw, True)
         return copied
 
